#include <uhub.h>

static struct ip_addr_encap ip4_a;
static struct ip_addr_encap ip4_b;
static struct ip_addr_encap ip4_c;
static struct ip_addr_encap ip6_a;
static struct ip_addr_encap ip6_b;
static struct ip_addr_encap ip6_c;
static struct ip_addr_encap mask;
static struct ip_ban_record ban6;
static struct ip_ban_record ban4;

EXO_TEST(create_addresses_1, {
	return
		ip_convert_to_binary("192.168.0.0",     &ip4_a) &&
		ip_convert_to_binary("192.168.255.255", &ip4_b) &&
		ip_convert_to_binary("192.168.0.1",     &ip4_c);
});

EXO_TEST(create_addresses_2, {
	return
		ip_convert_to_binary("2001::201:2ff:fefa:0",    &ip6_a) &&
		ip_convert_to_binary("2001::201:2ff:fefa:ffff", &ip6_b) &&
		ip_convert_to_binary("2001::201:2ff:fefa:fffe", &ip6_c);
});

EXO_TEST(ip_is_valid_ipv4_1, {
	return ip_is_valid_ipv4("127.0.0.1");
});

EXO_TEST(ip_is_valid_ipv4_2, {
	return ip_is_valid_ipv4("10.18.1.178");
});

EXO_TEST(ip_is_valid_ipv4_3, {
	return ip_is_valid_ipv4("10.18.1.178");
});

EXO_TEST(ip_is_valid_ipv4_4, {
	return ip_is_valid_ipv4("224.0.0.1");
});

EXO_TEST(ip_is_valid_ipv4_5, {
	return !ip_is_valid_ipv4("224.0.0.");
});

EXO_TEST(ip_is_valid_ipv4_6, {
	return !ip_is_valid_ipv4("invalid");
});

EXO_TEST(ip_is_valid_ipv4_7, {
	return !ip_is_valid_ipv4("localhost");
});

EXO_TEST(ip_is_valid_ipv4_8, {
	return !ip_is_valid_ipv4("123.45.67.890");
});

EXO_TEST(ip_is_valid_ipv4_9, {
	return !ip_is_valid_ipv4("777.777.777.777");
});

EXO_TEST(ip_is_valid_ipv6_1, {
	return ip_is_valid_ipv6("::");
});

EXO_TEST(ip_is_valid_ipv6_2, {
	return ip_is_valid_ipv6("::1");
});

EXO_TEST(ip_is_valid_ipv6_3, {
	return ip_is_valid_ipv6("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff");
});

EXO_TEST(ip4_compare_1, {
	return ip_compare(&ip4_a, &ip4_b) < 0;
});

EXO_TEST(ip4_compare_2, {
	return ip_compare(&ip4_a, &ip4_c) < 0;
});

EXO_TEST(ip4_compare_3, {
	return ip_compare(&ip4_b, &ip4_c) > 0;
});

EXO_TEST(ip4_compare_4, {
	return ip_compare(&ip4_b, &ip4_a) > 0;
});

EXO_TEST(ip4_compare_5, {
	return ip_compare(&ip4_c, &ip4_a) > 0;
});

EXO_TEST(ip4_compare_6, {
	return ip_compare(&ip4_c, &ip4_c) == 0;
});


EXO_TEST(ip6_compare_1, {
	return ip_compare(&ip6_a, &ip6_b) < 0;
});

EXO_TEST(ip6_compare_2, {
	return ip_compare(&ip6_a, &ip6_c) < 0;
});

EXO_TEST(ip6_compare_3, {
	return ip_compare(&ip6_b, &ip6_c) > 0;
});

EXO_TEST(ip6_compare_4, {
	return ip_compare(&ip6_b, &ip6_a) > 0;
});

EXO_TEST(ip6_compare_5, {
	return ip_compare(&ip6_c, &ip6_a) > 0;
});

EXO_TEST(ip6_compare_6, {
	return ip_compare(&ip6_c, &ip6_c) == 0;
});


#define LMASK4(bits)    !ip_mask_create_left (AF_INET,  bits, &mask)
#define LMASK6(bits)    !ip_mask_create_left (AF_INET6, bits, &mask)
#define RMASK4(bits)    !ip_mask_create_right(AF_INET,  bits, &mask)
#define RMASK6(bits)    !ip_mask_create_right(AF_INET6, bits, &mask)
#define CHECK(expect)   !strcmp(ip_convert_to_string(&mask), expect)

/* Check IPv4 masks */
EXO_TEST(ipv4_lmask_create_0,  { return LMASK4( 0) && CHECK("0.0.0.0"); });
EXO_TEST(ipv4_lmask_create_1,  { return LMASK4( 1) && CHECK("128.0.0.0"); });
EXO_TEST(ipv4_lmask_create_2,  { return LMASK4( 2) && CHECK("192.0.0.0"); });
EXO_TEST(ipv4_lmask_create_3,  { return LMASK4( 3) && CHECK("224.0.0.0"); });
EXO_TEST(ipv4_lmask_create_4,  { return LMASK4( 4) && CHECK("240.0.0.0"); });
EXO_TEST(ipv4_lmask_create_5,  { return LMASK4( 5) && CHECK("248.0.0.0"); });
EXO_TEST(ipv4_lmask_create_6,  { return LMASK4( 6) && CHECK("252.0.0.0"); });
EXO_TEST(ipv4_lmask_create_7,  { return LMASK4( 7) && CHECK("254.0.0.0"); });
EXO_TEST(ipv4_lmask_create_8,  { return LMASK4( 8) && CHECK("255.0.0.0"); });
EXO_TEST(ipv4_lmask_create_9,  { return LMASK4( 9) && CHECK("255.128.0.0"); });
EXO_TEST(ipv4_lmask_create_10, { return LMASK4(10) && CHECK("255.192.0.0"); });
EXO_TEST(ipv4_lmask_create_11, { return LMASK4(11) && CHECK("255.224.0.0"); });
EXO_TEST(ipv4_lmask_create_12, { return LMASK4(12) && CHECK("255.240.0.0"); });
EXO_TEST(ipv4_lmask_create_13, { return LMASK4(13) && CHECK("255.248.0.0"); });
EXO_TEST(ipv4_lmask_create_14, { return LMASK4(14) && CHECK("255.252.0.0"); });
EXO_TEST(ipv4_lmask_create_15, { return LMASK4(15) && CHECK("255.254.0.0"); });
EXO_TEST(ipv4_lmask_create_16, { return LMASK4(16) && CHECK("255.255.0.0"); });
EXO_TEST(ipv4_lmask_create_17, { return LMASK4(17) && CHECK("255.255.128.0"); });
EXO_TEST(ipv4_lmask_create_18, { return LMASK4(18) && CHECK("255.255.192.0"); });
EXO_TEST(ipv4_lmask_create_19, { return LMASK4(19) && CHECK("255.255.224.0"); });
EXO_TEST(ipv4_lmask_create_20, { return LMASK4(20) && CHECK("255.255.240.0"); });
EXO_TEST(ipv4_lmask_create_21, { return LMASK4(21) && CHECK("255.255.248.0"); });
EXO_TEST(ipv4_lmask_create_22, { return LMASK4(22) && CHECK("255.255.252.0"); });
EXO_TEST(ipv4_lmask_create_23, { return LMASK4(23) && CHECK("255.255.254.0"); });
EXO_TEST(ipv4_lmask_create_24, { return LMASK4(24) && CHECK("255.255.255.0"); });
EXO_TEST(ipv4_lmask_create_25, { return LMASK4(25) && CHECK("255.255.255.128"); });
EXO_TEST(ipv4_lmask_create_26, { return LMASK4(26) && CHECK("255.255.255.192"); });
EXO_TEST(ipv4_lmask_create_27, { return LMASK4(27) && CHECK("255.255.255.224"); });
EXO_TEST(ipv4_lmask_create_28, { return LMASK4(28) && CHECK("255.255.255.240"); });
EXO_TEST(ipv4_lmask_create_29, { return LMASK4(29) && CHECK("255.255.255.248"); });
EXO_TEST(ipv4_lmask_create_30, { return LMASK4(30) && CHECK("255.255.255.252"); });
EXO_TEST(ipv4_lmask_create_31, { return LMASK4(31) && CHECK("255.255.255.254"); });
EXO_TEST(ipv4_lmask_create_32, { return LMASK4(32) && CHECK("255.255.255.255"); });

/* Check IPv4 right to left mask */
EXO_TEST(ipv4_rmask_create_0,  { return RMASK4( 0) && CHECK("0.0.0.0"); });
EXO_TEST(ipv4_rmask_create_1,  { return RMASK4( 1) && CHECK("0.0.0.1"); });
EXO_TEST(ipv4_rmask_create_2,  { return RMASK4( 2) && CHECK("0.0.0.3"); });
EXO_TEST(ipv4_rmask_create_3,  { return RMASK4( 3) && CHECK("0.0.0.7"); });
EXO_TEST(ipv4_rmask_create_4,  { return RMASK4( 4) && CHECK("0.0.0.15"); });
EXO_TEST(ipv4_rmask_create_5,  { return RMASK4( 5) && CHECK("0.0.0.31"); });
EXO_TEST(ipv4_rmask_create_6,  { return RMASK4( 6) && CHECK("0.0.0.63"); });
EXO_TEST(ipv4_rmask_create_7,  { return RMASK4( 7) && CHECK("0.0.0.127"); });
EXO_TEST(ipv4_rmask_create_8,  { return RMASK4( 8) && CHECK("0.0.0.255"); });
EXO_TEST(ipv4_rmask_create_9,  { return RMASK4( 9) && CHECK("0.0.1.255"); });
EXO_TEST(ipv4_rmask_create_10, { return RMASK4(10) && CHECK("0.0.3.255"); });
EXO_TEST(ipv4_rmask_create_11, { return RMASK4(11) && CHECK("0.0.7.255"); });
EXO_TEST(ipv4_rmask_create_12, { return RMASK4(12) && CHECK("0.0.15.255"); });
EXO_TEST(ipv4_rmask_create_13, { return RMASK4(13) && CHECK("0.0.31.255"); });
EXO_TEST(ipv4_rmask_create_14, { return RMASK4(14) && CHECK("0.0.63.255"); });
EXO_TEST(ipv4_rmask_create_15, { return RMASK4(15) && CHECK("0.0.127.255"); });
EXO_TEST(ipv4_rmask_create_16, { return RMASK4(16) && CHECK("0.0.255.255"); });
EXO_TEST(ipv4_rmask_create_17, { return RMASK4(17) && CHECK("0.1.255.255"); });
EXO_TEST(ipv4_rmask_create_18, { return RMASK4(18) && CHECK("0.3.255.255"); });
EXO_TEST(ipv4_rmask_create_19, { return RMASK4(19) && CHECK("0.7.255.255"); });
EXO_TEST(ipv4_rmask_create_20, { return RMASK4(20) && CHECK("0.15.255.255"); });
EXO_TEST(ipv4_rmask_create_21, { return RMASK4(21) && CHECK("0.31.255.255"); });
EXO_TEST(ipv4_rmask_create_22, { return RMASK4(22) && CHECK("0.63.255.255"); });
EXO_TEST(ipv4_rmask_create_23, { return RMASK4(23) && CHECK("0.127.255.255"); });
EXO_TEST(ipv4_rmask_create_24, { return RMASK4(24) && CHECK("0.255.255.255"); });
EXO_TEST(ipv4_rmask_create_25, { return RMASK4(25) && CHECK("1.255.255.255"); });
EXO_TEST(ipv4_rmask_create_26, { return RMASK4(26) && CHECK("3.255.255.255"); });
EXO_TEST(ipv4_rmask_create_27, { return RMASK4(27) && CHECK("7.255.255.255"); });
EXO_TEST(ipv4_rmask_create_28, { return RMASK4(28) && CHECK("15.255.255.255"); });
EXO_TEST(ipv4_rmask_create_29, { return RMASK4(29) && CHECK("31.255.255.255"); });
EXO_TEST(ipv4_rmask_create_30, { return RMASK4(30) && CHECK("63.255.255.255"); });
EXO_TEST(ipv4_rmask_create_31, { return RMASK4(31) && CHECK("127.255.255.255"); });
EXO_TEST(ipv4_rmask_create_32, { return RMASK4(32) && CHECK("255.255.255.255"); });


/* Check IPv6 masks */
EXO_TEST(ip6_lmask_create_0,   { return LMASK6(  0) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_lmask_create_1,   { return LMASK6(  1) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fffe"); });
EXO_TEST(ip6_lmask_create_2,   { return LMASK6(  2) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fffc"); });
EXO_TEST(ip6_lmask_create_3,   { return LMASK6(  3) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fff8"); });
EXO_TEST(ip6_lmask_create_4,   { return LMASK6(  4) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fff0"); });
EXO_TEST(ip6_lmask_create_5,   { return LMASK6(  5) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffe0"); });
EXO_TEST(ip6_lmask_create_6,   { return LMASK6(  6) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffc0"); });
EXO_TEST(ip6_lmask_create_7,   { return LMASK6(  7) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ff80"); });
EXO_TEST(ip6_lmask_create_8,   { return LMASK6(  8) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ff00"); });
EXO_TEST(ip6_lmask_create_9,   { return LMASK6(  9) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fe00"); });
EXO_TEST(ip6_lmask_create_10,  { return LMASK6( 10) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:fc00"); });
EXO_TEST(ip6_lmask_create_11,  { return LMASK6( 11) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:f800"); });
EXO_TEST(ip6_lmask_create_12,  { return LMASK6( 12) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:f000"); });
EXO_TEST(ip6_lmask_create_13,  { return LMASK6( 13) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:e000"); });
EXO_TEST(ip6_lmask_create_14,  { return LMASK6( 14) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:c000"); });
EXO_TEST(ip6_lmask_create_15,  { return LMASK6( 15) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:8000"); });
EXO_TEST(ip6_lmask_create_16,  { return LMASK6( 16) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:0"); });
EXO_TEST(ip6_lmask_create_17,  { return LMASK6( 17) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fffe:0"); });
EXO_TEST(ip6_lmask_create_18,  { return LMASK6( 18) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fffc:0"); });
EXO_TEST(ip6_lmask_create_19,  { return LMASK6( 19) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fff8:0"); });
EXO_TEST(ip6_lmask_create_20,  { return LMASK6( 20) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fff0:0"); });
EXO_TEST(ip6_lmask_create_21,  { return LMASK6( 21) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffe0:0"); });
EXO_TEST(ip6_lmask_create_22,  { return LMASK6( 22) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffc0:0"); });
EXO_TEST(ip6_lmask_create_23,  { return LMASK6( 23) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ff80:0"); });
EXO_TEST(ip6_lmask_create_24,  { return LMASK6( 24) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ff00:0"); });
EXO_TEST(ip6_lmask_create_25,  { return LMASK6( 25) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fe00:0"); });
EXO_TEST(ip6_lmask_create_26,  { return LMASK6( 26) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:fc00:0"); });
EXO_TEST(ip6_lmask_create_27,  { return LMASK6( 27) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:f800:0"); });
EXO_TEST(ip6_lmask_create_28,  { return LMASK6( 28) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:f000:0"); });
EXO_TEST(ip6_lmask_create_29,  { return LMASK6( 29) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:e000:0"); });
EXO_TEST(ip6_lmask_create_30,  { return LMASK6( 30) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:c000:0"); });
EXO_TEST(ip6_lmask_create_31,  { return LMASK6( 31) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:8000:0"); });
EXO_TEST(ip6_lmask_create_32,  { return LMASK6( 32) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff::"); });
EXO_TEST(ip6_lmask_create_33,  { return LMASK6( 33) && CHECK("ffff:ffff:ffff:ffff:ffff:fffe::"); });
EXO_TEST(ip6_lmask_create_34,  { return LMASK6( 34) && CHECK("ffff:ffff:ffff:ffff:ffff:fffc::"); });
EXO_TEST(ip6_lmask_create_35,  { return LMASK6( 35) && CHECK("ffff:ffff:ffff:ffff:ffff:fff8::"); });
EXO_TEST(ip6_lmask_create_36,  { return LMASK6( 36) && CHECK("ffff:ffff:ffff:ffff:ffff:fff0::"); });
EXO_TEST(ip6_lmask_create_37,  { return LMASK6( 37) && CHECK("ffff:ffff:ffff:ffff:ffff:ffe0::"); });
EXO_TEST(ip6_lmask_create_38,  { return LMASK6( 38) && CHECK("ffff:ffff:ffff:ffff:ffff:ffc0::"); });
EXO_TEST(ip6_lmask_create_39,  { return LMASK6( 39) && CHECK("ffff:ffff:ffff:ffff:ffff:ff80::"); });
EXO_TEST(ip6_lmask_create_40,  { return LMASK6( 40) && CHECK("ffff:ffff:ffff:ffff:ffff:ff00::"); });
EXO_TEST(ip6_lmask_create_41,  { return LMASK6( 41) && CHECK("ffff:ffff:ffff:ffff:ffff:fe00::"); });
EXO_TEST(ip6_lmask_create_42,  { return LMASK6( 42) && CHECK("ffff:ffff:ffff:ffff:ffff:fc00::"); });
EXO_TEST(ip6_lmask_create_43,  { return LMASK6( 43) && CHECK("ffff:ffff:ffff:ffff:ffff:f800::"); });
EXO_TEST(ip6_lmask_create_44,  { return LMASK6( 44) && CHECK("ffff:ffff:ffff:ffff:ffff:f000::"); });
EXO_TEST(ip6_lmask_create_45,  { return LMASK6( 45) && CHECK("ffff:ffff:ffff:ffff:ffff:e000::"); });
EXO_TEST(ip6_lmask_create_46,  { return LMASK6( 46) && CHECK("ffff:ffff:ffff:ffff:ffff:c000::"); });
EXO_TEST(ip6_lmask_create_47,  { return LMASK6( 47) && CHECK("ffff:ffff:ffff:ffff:ffff:8000::"); });
EXO_TEST(ip6_lmask_create_48,  { return LMASK6( 48) && CHECK("ffff:ffff:ffff:ffff:ffff::"); });
EXO_TEST(ip6_lmask_create_49,  { return LMASK6( 49) && CHECK("ffff:ffff:ffff:ffff:fffe::"); });
EXO_TEST(ip6_lmask_create_50,  { return LMASK6( 50) && CHECK("ffff:ffff:ffff:ffff:fffc::"); });
EXO_TEST(ip6_lmask_create_51,  { return LMASK6( 51) && CHECK("ffff:ffff:ffff:ffff:fff8::"); });
EXO_TEST(ip6_lmask_create_52,  { return LMASK6( 52) && CHECK("ffff:ffff:ffff:ffff:fff0::"); });
EXO_TEST(ip6_lmask_create_53,  { return LMASK6( 53) && CHECK("ffff:ffff:ffff:ffff:ffe0::"); });
EXO_TEST(ip6_lmask_create_54,  { return LMASK6( 54) && CHECK("ffff:ffff:ffff:ffff:ffc0::"); });
EXO_TEST(ip6_lmask_create_55,  { return LMASK6( 55) && CHECK("ffff:ffff:ffff:ffff:ff80::"); });
EXO_TEST(ip6_lmask_create_56,  { return LMASK6( 56) && CHECK("ffff:ffff:ffff:ffff:ff00::"); });
EXO_TEST(ip6_lmask_create_57,  { return LMASK6( 57) && CHECK("ffff:ffff:ffff:ffff:fe00::"); });
EXO_TEST(ip6_lmask_create_58,  { return LMASK6( 58) && CHECK("ffff:ffff:ffff:ffff:fc00::"); });
EXO_TEST(ip6_lmask_create_59,  { return LMASK6( 59) && CHECK("ffff:ffff:ffff:ffff:f800::"); });
EXO_TEST(ip6_lmask_create_60,  { return LMASK6( 60) && CHECK("ffff:ffff:ffff:ffff:f000::"); });
EXO_TEST(ip6_lmask_create_61,  { return LMASK6( 61) && CHECK("ffff:ffff:ffff:ffff:e000::"); });
EXO_TEST(ip6_lmask_create_62,  { return LMASK6( 62) && CHECK("ffff:ffff:ffff:ffff:c000::"); });
EXO_TEST(ip6_lmask_create_63,  { return LMASK6( 63) && CHECK("ffff:ffff:ffff:ffff:8000::"); });
EXO_TEST(ip6_lmask_create_64,  { return LMASK6( 64) && CHECK("ffff:ffff:ffff:ffff::"); });
EXO_TEST(ip6_lmask_create_65,  { return LMASK6( 65) && CHECK("ffff:ffff:ffff:fffe::"); });
EXO_TEST(ip6_lmask_create_66,  { return LMASK6( 66) && CHECK("ffff:ffff:ffff:fffc::"); });
EXO_TEST(ip6_lmask_create_67,  { return LMASK6( 67) && CHECK("ffff:ffff:ffff:fff8::"); });
EXO_TEST(ip6_lmask_create_68,  { return LMASK6( 68) && CHECK("ffff:ffff:ffff:fff0::"); });
EXO_TEST(ip6_lmask_create_69,  { return LMASK6( 69) && CHECK("ffff:ffff:ffff:ffe0::"); });
EXO_TEST(ip6_lmask_create_70,  { return LMASK6( 70) && CHECK("ffff:ffff:ffff:ffc0::"); });
EXO_TEST(ip6_lmask_create_71,  { return LMASK6( 71) && CHECK("ffff:ffff:ffff:ff80::"); });
EXO_TEST(ip6_lmask_create_72,  { return LMASK6( 72) && CHECK("ffff:ffff:ffff:ff00::"); });
EXO_TEST(ip6_lmask_create_73,  { return LMASK6( 73) && CHECK("ffff:ffff:ffff:fe00::"); });
EXO_TEST(ip6_lmask_create_74,  { return LMASK6( 74) && CHECK("ffff:ffff:ffff:fc00::"); });
EXO_TEST(ip6_lmask_create_75,  { return LMASK6( 75) && CHECK("ffff:ffff:ffff:f800::"); });
EXO_TEST(ip6_lmask_create_76,  { return LMASK6( 76) && CHECK("ffff:ffff:ffff:f000::"); });
EXO_TEST(ip6_lmask_create_77,  { return LMASK6( 77) && CHECK("ffff:ffff:ffff:e000::"); });
EXO_TEST(ip6_lmask_create_78,  { return LMASK6( 78) && CHECK("ffff:ffff:ffff:c000::"); });
EXO_TEST(ip6_lmask_create_79,  { return LMASK6( 79) && CHECK("ffff:ffff:ffff:8000::"); });
EXO_TEST(ip6_lmask_create_80,  { return LMASK6( 80) && CHECK("ffff:ffff:ffff::"); });
EXO_TEST(ip6_lmask_create_81,  { return LMASK6( 81) && CHECK("ffff:ffff:fffe::"); });
EXO_TEST(ip6_lmask_create_82,  { return LMASK6( 82) && CHECK("ffff:ffff:fffc::"); });
EXO_TEST(ip6_lmask_create_83,  { return LMASK6( 83) && CHECK("ffff:ffff:fff8::"); });
EXO_TEST(ip6_lmask_create_84,  { return LMASK6( 84) && CHECK("ffff:ffff:fff0::"); });
EXO_TEST(ip6_lmask_create_85,  { return LMASK6( 85) && CHECK("ffff:ffff:ffe0::"); });
EXO_TEST(ip6_lmask_create_86,  { return LMASK6( 86) && CHECK("ffff:ffff:ffc0::"); });
EXO_TEST(ip6_lmask_create_87,  { return LMASK6( 87) && CHECK("ffff:ffff:ff80::"); });
EXO_TEST(ip6_lmask_create_88,  { return LMASK6( 88) && CHECK("ffff:ffff:ff00::"); });
EXO_TEST(ip6_lmask_create_89,  { return LMASK6( 89) && CHECK("ffff:ffff:fe00::"); });
EXO_TEST(ip6_lmask_create_90,  { return LMASK6( 90) && CHECK("ffff:ffff:fc00::"); });
EXO_TEST(ip6_lmask_create_91,  { return LMASK6( 91) && CHECK("ffff:ffff:f800::"); });
EXO_TEST(ip6_lmask_create_92,  { return LMASK6( 92) && CHECK("ffff:ffff:f000::"); });
EXO_TEST(ip6_lmask_create_93,  { return LMASK6( 93) && CHECK("ffff:ffff:e000::"); });
EXO_TEST(ip6_lmask_create_94,  { return LMASK6( 94) && CHECK("ffff:ffff:c000::"); });
EXO_TEST(ip6_lmask_create_95,  { return LMASK6( 95) && CHECK("ffff:ffff:8000::"); });
EXO_TEST(ip6_lmask_create_96,  { return LMASK6( 96) && CHECK("ffff:ffff::"); });
EXO_TEST(ip6_lmask_create_97,  { return LMASK6( 97) && CHECK("ffff:fffe::"); });
EXO_TEST(ip6_lmask_create_98,  { return LMASK6( 98) && CHECK("ffff:fffc::"); });
EXO_TEST(ip6_lmask_create_99,  { return LMASK6( 99) && CHECK("ffff:fff8::"); });
EXO_TEST(ip6_lmask_create_100, { return LMASK6(100) && CHECK("ffff:fff0::"); });
EXO_TEST(ip6_lmask_create_101, { return LMASK6(101) && CHECK("ffff:ffe0::"); });
EXO_TEST(ip6_lmask_create_102, { return LMASK6(102) && CHECK("ffff:ffc0::"); });
EXO_TEST(ip6_lmask_create_103, { return LMASK6(103) && CHECK("ffff:ff80::"); });
EXO_TEST(ip6_lmask_create_104, { return LMASK6(104) && CHECK("ffff:ff00::"); });
EXO_TEST(ip6_lmask_create_105, { return LMASK6(105) && CHECK("ffff:fe00::"); });
EXO_TEST(ip6_lmask_create_106, { return LMASK6(106) && CHECK("ffff:fc00::"); });
EXO_TEST(ip6_lmask_create_107, { return LMASK6(107) && CHECK("ffff:f800::"); });
EXO_TEST(ip6_lmask_create_108, { return LMASK6(108) && CHECK("ffff:f000::"); });
EXO_TEST(ip6_lmask_create_109, { return LMASK6(109) && CHECK("ffff:e000::"); });
EXO_TEST(ip6_lmask_create_110, { return LMASK6(110) && CHECK("ffff:c000::"); });
EXO_TEST(ip6_lmask_create_111, { return LMASK6(111) && CHECK("ffff:8000::"); });
EXO_TEST(ip6_lmask_create_112, { return LMASK6(112) && CHECK("ffff::"); });
EXO_TEST(ip6_lmask_create_113, { return LMASK6(113) && CHECK("fffe::"); });
EXO_TEST(ip6_lmask_create_114, { return LMASK6(114) && CHECK("fffc::"); });
EXO_TEST(ip6_lmask_create_115, { return LMASK6(115) && CHECK("fff8::"); });
EXO_TEST(ip6_lmask_create_116, { return LMASK6(116) && CHECK("fff0::"); });
EXO_TEST(ip6_lmask_create_117, { return LMASK6(117) && CHECK("ffe0::"); });
EXO_TEST(ip6_lmask_create_118, { return LMASK6(118) && CHECK("ffc0::"); });
EXO_TEST(ip6_lmask_create_119, { return LMASK6(119) && CHECK("ff80::"); });
EXO_TEST(ip6_lmask_create_120, { return LMASK6(120) && CHECK("ff00::"); });
EXO_TEST(ip6_lmask_create_121, { return LMASK6(121) && CHECK("fe00::"); });
EXO_TEST(ip6_lmask_create_122, { return LMASK6(122) && CHECK("fc00::"); });
EXO_TEST(ip6_lmask_create_123, { return LMASK6(123) && CHECK("f800::"); });
EXO_TEST(ip6_lmask_create_124, { return LMASK6(124) && CHECK("f000::"); });
EXO_TEST(ip6_lmask_create_125, { return LMASK6(125) && CHECK("e000::"); });
EXO_TEST(ip6_lmask_create_126, { return LMASK6(126) && CHECK("c000::"); });
EXO_TEST(ip6_lmask_create_127, { return LMASK6(127) && CHECK("8000::"); });
EXO_TEST(ip6_lmask_create_128, { return LMASK6(128) && CHECK("::"); });

/* Check IPv6 right to left masks */
EXO_TEST(ip6_rmask_create_0,   { return RMASK6(  0) && CHECK("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_1,   { return RMASK6(  1) && CHECK("7fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_2,   { return RMASK6(  2) && CHECK("3fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_3,   { return RMASK6(  3) && CHECK("1fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_4,   { return RMASK6(  4) && CHECK("fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_5,   { return RMASK6(  5) && CHECK("7ff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_6,   { return RMASK6(  6) && CHECK("3ff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_7,   { return RMASK6(  7) && CHECK("1ff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_8,   { return RMASK6(  8) && CHECK("ff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_9,   { return RMASK6(  9) && CHECK("7f:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_10,  { return RMASK6( 10) && CHECK("3f:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_11,  { return RMASK6( 11) && CHECK("1f:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_12,  { return RMASK6( 12) && CHECK("f:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_13,  { return RMASK6( 13) && CHECK("7:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_14,  { return RMASK6( 14) && CHECK("3:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_15,  { return RMASK6( 15) && CHECK("1:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_16,  { return RMASK6( 16) && CHECK("0:ffff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_17,  { return RMASK6( 17) && CHECK("0:7fff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_18,  { return RMASK6( 18) && CHECK("0:3fff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_19,  { return RMASK6( 19) && CHECK("0:1fff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_20,  { return RMASK6( 20) && CHECK("0:fff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_21,  { return RMASK6( 21) && CHECK("0:7ff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_22,  { return RMASK6( 22) && CHECK("0:3ff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_23,  { return RMASK6( 23) && CHECK("0:1ff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_24,  { return RMASK6( 24) && CHECK("0:ff:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_25,  { return RMASK6( 25) && CHECK("0:7f:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_26,  { return RMASK6( 26) && CHECK("0:3f:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_27,  { return RMASK6( 27) && CHECK("0:1f:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_28,  { return RMASK6( 28) && CHECK("0:f:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_29,  { return RMASK6( 29) && CHECK("0:7:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_30,  { return RMASK6( 30) && CHECK("0:3:ffff:ffff:ffff:ffff:ffff:ffff"); });
EXO_TEST(ip6_rmask_create_31,  { return RMASK6( 31) && CHECK("0:1:ffff:ffff:ffff:ffff:ffff:ffff"); });

EXO_TEST(check_ban_setup_1, {
	return	ip_convert_to_binary("2001::201:2ff:fefa:0",    &ban6.lo) &&
			ip_convert_to_binary("2001::201:2ff:fefa:ffff", &ban6.hi) &&
			ip_convert_to_binary("192.168.0.0",             &ban4.lo) &&
			ip_convert_to_binary("192.168.0.255",           &ban4.hi);
});

EXO_TEST(check_ban_ipv4_1, {
	struct ip_addr_encap addr; ip_convert_to_binary("192.168.0.0", &addr);
	return acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_ipv4_2, {
	struct ip_addr_encap addr; ip_convert_to_binary("192.168.0.1", &addr);
	return acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_ipv4_3, {
	struct ip_addr_encap addr; ip_convert_to_binary("192.168.0.255", &addr);
	return acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_ipv4_4, {
	struct ip_addr_encap addr; ip_convert_to_binary("192.168.1.0", &addr);
	return !acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_ipv4_5, {
	struct ip_addr_encap addr; ip_convert_to_binary("192.167.255.255", &addr);
	return !acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_ipv6_1, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fefa:0", &addr);
	return acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_ipv6_2, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fefa:1", &addr);
	return acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_ipv6_3, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fefa:fffe", &addr);
	return acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_ipv6_4, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fefa:ffff", &addr);
	return acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_ipv6_5, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fefb:0", &addr);
	return !acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_ipv6_6, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fef9:ffff", &addr);
	return !acl_check_ban(&addr, &ban6);
});

EXO_TEST(check_ban_afmix_1, {
	struct ip_addr_encap addr; ip_convert_to_binary("2001::201:2ff:fef9:ffff", &addr);
	return !acl_check_ban(&addr, &ban4);
});

EXO_TEST(check_ban_afmix_2, {
	struct ip_addr_encap addr; ip_convert_to_binary("10.20.30.40", &addr);
	return !acl_check_ban(&addr, &ban6);
});

EXO_TEST(ip4_bitwise_AND_1, {
	ip_convert_to_binary("255.255.255.255", &ip4_a);
	ip_convert_to_binary("255.255.255.0",   &ip4_b);
	ip_mask_apply_AND(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "255.255.255.0");
});

EXO_TEST(ip4_bitwise_AND_2, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.255.255.0",   &ip4_b);
	ip_mask_apply_AND(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "192.168.217.0");
});

EXO_TEST(ip4_bitwise_AND_3, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.255.0.0",     &ip4_b);
	ip_mask_apply_AND(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "192.168.0.0");
});

EXO_TEST(ip4_bitwise_AND_4, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.0.0.0",       &ip4_b);
	ip_mask_apply_AND(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "192.0.0.0");
});

EXO_TEST(ip4_bitwise_AND_5, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("0.0.0.0",   &ip4_b);
	ip_mask_apply_AND(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "0.0.0.0");
});

EXO_TEST(ip4_bitwise_OR_1, {
	ip_convert_to_binary("255.255.255.255", &ip4_a);
	ip_convert_to_binary("255.255.255.0",   &ip4_b);
	ip_mask_apply_OR(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "255.255.255.255");
});

EXO_TEST(ip4_bitwise_OR_2, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.255.255.0",   &ip4_b);
	ip_mask_apply_OR(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "255.255.255.113");
});

EXO_TEST(ip4_bitwise_OR_3, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.255.0.0",     &ip4_b);
	ip_mask_apply_OR(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "255.255.217.113");
});

EXO_TEST(ip4_bitwise_OR_4, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("255.0.0.0",       &ip4_b);
	ip_mask_apply_OR(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "255.168.217.113");
});

EXO_TEST(ip4_bitwise_OR_5, {
	ip_convert_to_binary("192.168.217.113", &ip4_a);
	ip_convert_to_binary("0.0.0.0",         &ip4_b);
	ip_mask_apply_OR(&ip4_a, &ip4_b, &ip4_c);
	return !strcmp(ip_convert_to_string(&ip4_c), "192.168.217.113");
});

EXO_TEST(ip6_bitwise_AND_1, {
	ip_convert_to_binary("7f7f:ffff:ffff:3f3f:ffff:ffff:ffff:ffff", &ip6_a);
	ip_convert_to_binary("ffff:ffff:ffff:ffff:ffff:ffff:ffff:0",    &ip6_b);
	ip_mask_apply_AND(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "7f7f:ffff:ffff:3f3f:ffff:ffff:ffff:0");
});

EXO_TEST(ip6_bitwise_AND_2, {
	ip_convert_to_binary("7777:cccc:3333:1111:ffff:ffff:ffff:ffff", &ip6_a);
	ip_convert_to_binary("ffff:ffff:ffff:ffff::",    &ip6_b);
	ip_mask_apply_AND(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "7777:cccc:3333:1111::");
});

EXO_TEST(ip6_bitwise_AND_3, {
	ip_convert_to_binary("7777:cccc:3333:1111:ffff:ffff:ffff:ffff", &ip6_a);
	ip_convert_to_binary("::",    &ip6_b);
	ip_mask_apply_AND(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "::");
});

EXO_TEST(ip6_bitwise_OR_1, {
	ip_convert_to_binary("7f7f:ffff:ffff:3f3f:ffff:ffff:ffff:ffff", &ip6_a);
	ip_convert_to_binary("ffff:ffff:ffff:ffff:ffff:ffff:ffff:0",    &ip6_b);
	ip_mask_apply_OR(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff");
});

EXO_TEST(ip6_bitwise_OR_2, {
	ip_convert_to_binary("7777:cccc:3333:1111:ffff:ffff:ffff:ffff", &ip6_a);
	ip_convert_to_binary("ffff:ffff:ffff:ffff:ffff:ffff:ffff:0",    &ip6_b);
	ip_mask_apply_OR(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff");
});

EXO_TEST(ip6_bitwise_OR_3, {
	ip_convert_to_binary("7777:cccc:3333:1111:ffff:ffff:ffff:1c1c", &ip6_a);
	ip_convert_to_binary("::",    &ip6_b);
	ip_mask_apply_OR(&ip6_a, &ip6_b, &ip6_c);
	return !strcmp(ip_convert_to_string(&ip6_c), "7777:cccc:3333:1111:ffff:ffff:ffff:1c1c");
});

