#include <uhub.h>

static struct user* g_user = 0;
static const char* test_string1 = "IINF AAfoo BBbar CCwhat\n";
static const char* test_string2 = "BMSG AAAB Hello\\sWorld!\n";

static void create_test_user()
{
	if (g_user)
		return;

	g_user = (struct user*) malloc(sizeof(struct user));
	memset(g_user, 0, sizeof(struct user));
	memcpy(g_user->nick, "exotic-tester", 13);
	g_user->sid = 1;
}

EXO_TEST(adc_message_parse_1, {
	struct adc_message* msg = adc_msg_create("IMSG Hello\\sWorld!");
	return msg != NULL;
});

EXO_TEST(adc_message_parse_2, {
	struct adc_message* msg = adc_msg_create(test_string2);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_3, {
	create_test_user();
	struct adc_message* msg = adc_msg_parse(g_user, "BMSG AAAB Hello\\sWorld!", 23);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_4, {
	struct adc_message* msg = adc_msg_parse(g_user, "BMSG AAAC Hello\\sWorld!", 23);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_5, {
	struct adc_message* msg = adc_msg_parse(g_user, "BMSG AAAB Hello\\sWorld!\n", 24);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_6, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +TCP4 Hello\\sWorld!\n", 30);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_7, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB -TCP4 Hello\\sWorld!\n", 30);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_8, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +TCP4+UDP4 Hello\\sWorld!\n", 35);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_9, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +TCP4-UDP4 Hello\\sWorld!\n", 35);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_10, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB -TCP4-UDP4 Hello\\sWorld!\n", 35);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_11, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB Hello\\sWorld!\n", 24);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_12, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB  Hello\\sWorld!\n", 25);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_13, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +jalla Hello\\sWorld!\n", 31);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_14, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +jall Hello\\sWorld!\n", 30);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_15, {
	struct adc_message* msg = adc_msg_parse(g_user, "FMSG AAAB +TCP4 Hello\\sWorld!\n", 30);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_16, {
	struct adc_message* msg = adc_msg_parse(g_user, "BMSG AAAB Hello\\sWorld!\n", 24);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_17, {
	struct adc_message* msg = adc_msg_parse(g_user, "BMSG aaab Hello\\sWorld!\n", 24);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_18, {
	struct adc_message* msg = adc_msg_parse(g_user, "DMSG AAAB AAAC Hello\\sthere!\n", 29);
	return msg != NULL;
});

EXO_TEST(adc_message_parse_19, {
	struct adc_message* msg = adc_msg_parse(g_user, "DMSG AAAC AAAB Hello\\sthere!\n", 29);
	return msg == NULL;
});

EXO_TEST(adc_message_parse_20, {
	struct adc_message* msg = adc_msg_parse(g_user, "EMSG AAAB AAAC Hello\\sthere!\n", 29);
	return msg != NULL;
});


EXO_TEST(adc_message_parse_21, {
	struct adc_message* msg = adc_msg_parse(g_user, "EMSG AAAC AAAB Hello\\sthere!\n", 29);
	return msg == NULL;
});

EXO_TEST(adc_message_add_arg_1, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_add_argument(msg, "XXwtf?");
	return strcmp(msg->cache, "IINF AAfoo BBbar CCwhat XXwtf?\n") == 0;
});

EXO_TEST(adc_message_add_arg_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_add_named_argument(msg, "XX", "wtf?");
	return strcmp(msg->cache, "IINF AAfoo BBbar CCwhat XXwtf?\n") == 0;
});

EXO_TEST(adc_message_remove_arg_1, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "AA");
	return strcmp(msg->cache, "IINF BBbar CCwhat\n") == 0;
});

EXO_TEST(adc_message_remove_arg_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "BB");
	return strcmp(msg->cache, "IINF AAfoo CCwhat\n") == 0;
});

EXO_TEST(adc_message_remove_arg_3, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "CC");
	return strcmp(msg->cache, "IINF AAfoo BBbar\n") == 0;
});

EXO_TEST(adc_message_replace_arg_1, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "AA");
	return strcmp(msg->cache, "IINF BBbar CCwhat\n") == 0;
});

EXO_TEST(adc_message_replace_arg_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "BB");
	return strcmp(msg->cache, "IINF AAfoo CCwhat\n") == 0;
});

EXO_TEST(adc_message_replace_arg_3, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_remove_named_argument(msg, "CC");
	return strcmp(msg->cache, "IINF AAfoo BBbar\n") == 0;
});


EXO_TEST(adc_message_get_arg_1, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_argument(msg, 0);
	return strcmp(c, "AAfoo") == 0;
});

EXO_TEST(adc_message_get_arg_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_argument(msg, 1);
	return strcmp(c, "BBbar") == 0;
});

EXO_TEST(adc_message_get_arg_3, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_argument(msg, 2);
	return strcmp(c, "CCwhat") == 0;
});

EXO_TEST(adc_message_get_arg_4, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_argument(msg, 3);
	return c == 0;
});

EXO_TEST(adc_message_get_named_arg_1, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_named_argument(msg, "AA");
	return strcmp(c, "foo") == 0;
});

EXO_TEST(adc_message_get_named_arg_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_named_argument(msg, "BB");
	return strcmp(c, "bar") == 0;
});

EXO_TEST(adc_message_get_named_arg_3, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_named_argument(msg, "CC");
	return strcmp(c, "what") == 0;
});

EXO_TEST(adc_message_get_named_arg_4, {
	struct adc_message* msg = adc_msg_create(test_string1);
	char* c = adc_msg_get_named_argument(msg, "XX");
	return c == 0;
});

EXO_TEST(adc_message_terminate_1, {
	struct adc_message* msg = adc_msg_create("IINF AAfoo BBbar CCwhat");
	adc_msg_unterminate_command(msg);
	return strcmp(msg->cache, "IINF AAfoo BBbar CCwhat") == 0;
	
});

EXO_TEST(adc_message_terminate_2, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_unterminate_command(msg);
	adc_msg_terminate_command(msg);
	return strcmp(msg->cache, "IINF AAfoo BBbar CCwhat\n") == 0;
});

EXO_TEST(adc_message_terminate_3, {
	struct adc_message* msg = adc_msg_create(test_string1);
	adc_msg_unterminate_command(msg);
	adc_msg_terminate_command(msg);
	adc_msg_unterminate_command(msg);
	return strcmp(msg->cache, "IINF AAfoo BBbar CCwhat") == 0;
});


EXO_TEST(adc_message_escape_1, {
	char* s = adc_msg_escape(test_string1);
	return strcmp(s, "IINF\\sAAfoo\\sBBbar\\sCCwhat\\n") == 0;
});

EXO_TEST(adc_message_escape_2, {
	char* s = adc_msg_escape(test_string1);
	char* s2 = adc_msg_unescape(s);
	return strcmp(s2, test_string1) == 0;
});

EXO_TEST(adc_message_escape_3, {
	char* s = adc_msg_unescape(test_string1);
	return strcmp(s, test_string1) == 0;
});

EXO_TEST(adc_message_copy_1, {
	struct adc_message* msg1 = adc_msg_create(test_string1);
	struct adc_message* msg2 = adc_msg_copy(msg1);
	return strncmp(msg1->cache, msg2->cache, msg1->length) == 0;
});

EXO_TEST(adc_message_copy_2, {
	struct adc_message* msg1 = adc_msg_parse(g_user, test_string2, strlen(test_string2));
	struct adc_message* msg2 = adc_msg_copy(msg1);
	return msg1->source == msg2->source;
});

EXO_TEST(adc_message_copy_3, {
	struct adc_message* msg1 = adc_msg_parse(g_user, test_string2, strlen(test_string2));
	struct adc_message* msg2 = adc_msg_copy(msg1);
	return (msg1->cmd        == msg2->cmd &&
			msg1->source     == msg2->source &&
			msg1->target     == msg2->target &&
			msg1->length     == msg2->length &&
			msg1->arg_offset == msg2->arg_offset &&
			msg1->priority   == msg2->priority &&
			msg1->capacity   == msg2->capacity && /* might not be true! */
			strcmp(msg1->cache, msg2->cache) == 0);
});

EXO_TEST(adc_message_copy_4, {
	struct adc_message* msg1 = adc_msg_parse(g_user, test_string2, strlen(test_string2));
	struct adc_message* msg2 = adc_msg_copy(msg1);
	return msg2->target == 0;
});


static struct adc_message* updater1 = NULL;
static struct adc_message* updater2 = NULL;
static const char* update_info1 = "BINF AAAB IDABCDEFGHIJKLMNOPQRSTUVWXYZ1234567ABCDEF NItester SL10 SS12817126127 SF4125 HN3 HR0 HO0 VE++\\s0.698 US104857600 DS81911808 SUTCP4,UDP4 I4127.0.0.1\n";
static const char* update_info2 = "BINF AAAB HN34\n";

EXO_TEST(adc_message_update_1, {
	updater1 = adc_msg_parse(g_user, update_info1, strlen(update_info1));
	return updater1 != NULL;
});

EXO_TEST(adc_message_update_2, {
	user_set_info(g_user, updater1);
	return g_user->info == updater1;
});

EXO_TEST(adc_message_update_3, {
	updater2 = adc_msg_parse(g_user, update_info2, strlen(update_info2));
	return updater2 != NULL;
});


extern void update_user_info(struct user* u, struct adc_message* cmd);

EXO_TEST(adc_message_update_4, {
	update_user_info(g_user, updater2);
	return strlen(g_user->info->cache) == 159;
});
