/*
 * uhub - A tiny ADC p2p connection hub
 * Copyright (C) 2007, Jan Vidar Krey
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef HAVE_UHUB_CONFIG_H
#define HAVE_UHUB_CONFIG_H

struct hub_config
{
	int server_port;             /**<<< "Server port to bind to (default: 1511)" */
	char* server_bind_addr;      /**<<< "Server bind address (default: '0.0.0.0' or '::')" */
	int hub_enabled;             /**<<< "Is server enabled (default: 1)" */
	int show_banner;             /**<<< "Show banner on connect (default: 1)" */
	int max_users;               /**<<< "Maximum number of users allowed on the hub (default: 500)" */
	int registered_users_only;   /**<<< "Allow registered users only (default: 0)" */
	int chat_only;               /**<<< "Allow chat only operation on hub (default: 0)" */
	char* file_motd;             /**<<< "File containing the 'message of the day' (default: '' - no motd)" */
	char* file_acl;              /**<<< "File containing user database (default: '' - no known users)" */
	char* hub_name;              /**<<< "Name of hub (default: 'My uhub hub')" */
	char* hub_description;       /**<<< "Name of hub (default: 'no description')" */
	int max_recv_buffer;         /**<<< "Max read buffer before parse, per user (default: 4096)" */
	int max_send_buffer;         /**<<< "Max send buffer before disconnect, per user (default: 16384)" */
	int max_send_buffer_soft;    /**<<< "Max send buffer before message drops, per user (default: 8192)" */
};

/**
 * This initializes the configuration variables, and sets the default
 * variables.
 *
 * NOTE: Any variable is set to it's default variable if zero.
 * This function is automatically called in read_config to set any
 * configuration that was missing there.
 */
extern void config_defaults(struct hub_config* config);

/**
 * Read configuration from file, and use the default variables for
 * the missing variables.
 *
 * @return -1 on error, 0 on success.
 */
extern int read_config(const char* file, struct hub_config* config);

/**
 * Free the configuration data (allocated by read_config, or config_defaults).
 */
extern void free_config(struct hub_config* config);




#endif /* HAVE_UHUB_CONFIG_H */

